export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.ShortName, "sincMA");

        this.addInput("N", InputType.int, 256);
        this.addInput("htrun", InputType.int, 3);
        this.addInput("Q", InputType.float, 0.5);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            sMA : this.addBuffer(),
        };
    }

    onInit() {
        this.buffers.sMA
            .setShape(Shape.Line)
            .setDrawingStyle(DrawingStyle.Solid)
            .setWidth(2)
            .setColor(Color.Blue);
    }

    async onUpdate() {
        const {sMA} = this.buffers,
            {N, htrun, Q, PriceType} = this.getInputs(),
            {Close} = await Bar.load([Bar.Mode.Close], PriceType),
            barSize = Close.length;


        let Fit = [], Dif = [], i, j, p, ss, pwr = 0, pwrMax, n,
            htRun = htrun,
            NVal = N
        ;

        if (NVal <= 0 || NVal > barSize) NVal = barSize;

        n = 1;
        while (n <= N) {
            pwr++;
            n *= 2;
        }
        pwr--;
        n = Math.pow(2, pwr);

        if (htRun <= 0)
            htRun = 0;

        if (htRun >= pwr)
            htRun = pwr;

        pwrMax = pwr - htrun;

        Fit.length = n+1;
        Dif.length = n+1;
        sMA.setSize(barSize);

        for (i = 0; i <= n; i++) {
            Fit[i] = (Close[n] - Close[0]) * i / n + Close[0];
            Dif[i] = Close[i] - Fit[i];
            sMA.set(i, Fit[i]);
        }

        ss = 2 * n;
        for (p = 1; p <= pwrMax; p++) {
            ss = ss / 2;
            for (i = 0; i <= n; i++) {
                Fit[i] = 0;
                for (j = 1; j <= n / ss; j++) {
                    Fit[i] += Dif[j * ss - ss / 2] * this.wave((2 * j - 1 - 2 * i / ss), Q);
                }
            }
            for (i = 0; i <= n; i++) {
                Dif[i] -= Fit[i];
                sMA.set(i, (sMA.get(i) + Fit[i]));
            }
        }


    }

    wave(x, Q) {
        if (x === 0)
            return (1);
        else
            return (Math.sin(Math.PI * x) / (Math.PI * x) / (x * x + 1) * (Q * x * x + 1));
    }
}